/*
 * UdfBurn's Library
 *
 * Copyright (C) 2020, KylinSoft Co., Ltd.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library.  If not, see <https://www.gnu.org/licenses/>.
 *
 * Authors: renpeijia <renpeijia@kylinos.cn>
 *
 */

#ifndef DISCCONTROL_H
#define DISCCONTROL_H

#include <QObject>
#include "udfburn_global.h"

class DiscControlImpl;

namespace UdfBurn {

class UDFBURN_EXPORT DiscControl : public QObject
{
    Q_OBJECT

public:
    DiscControl(QString device, QObject* parent = nullptr);
    ~DiscControl();
public:
    /**
     * @brief 读取光盘状态信息的入口
     * @return
     */
    bool work();

    /**
     * @brief 判断光盘是否支持udf格式
     * @return true 光盘支持udf格式化; false 光盘不支持
     */
    bool supportUdf();

    /**
     * @brief 判断光盘是否可以追加（目前使用在空盘或iso9660格式上）
     * @return true 光盘可以追加刻录； false 光盘不能追加刻录
     */
    bool discCanAppend();

    /**
     * @brief 判断光盘的文件系统是否是UDF
     * @return true udf文件系统； false 非udf文件系统
     */
    bool discIsUDF();

    /**
     * @brief 判断光盘是否符合所有R盘类型
     * @return true 光盘是R盘； false 光盘不是R盘
     */
    bool isAllRType();

    /**
     * @brief 获取光盘类型
     * @return 光盘类型，形如“DVD+RW”
     */
    QString discMediaType();

    /**
     * @brief 光盘卷标
     * @return 光盘卷标名称，形如“刻录”；若光盘未设置卷标，返回QString("")
     */
    QString discLabel();

    /**
     * @brief 获取光驱设备号
     * @return 光驱设备号，形如“/dev/sr0”
     */
    QString discDevice();

    /**
     * @brief 光驱是否被移除了
     * @return true 光驱已被移除; flase 光驱未被移除
     */
    bool isRemoved();

    /**
     * @brief 检测到光驱移除后，设置光驱被移除标识
     */
    void setRemoved(bool);

    /**
     * @brief 杀死不能正常结束的格式化进程
     */
    void killFormatProcess();

    /**
     * @brief 阻塞式Udf格式化操作接口，格式化成功后会立即弹出光盘
     * @param discLabel 卷标名
     * @return 格式化是否成功
     */
    bool formatUdfSync(QString discLabel);

    /**
     * @brief 阻塞式R盘Udf(1.02版本)追加刻录数据操作接口，刻录成功后会立即弹出光盘
     * @param discLabel 卷标名
     * @return R盘追加刻录是否成功
     */
    bool appendBurnDataUdfSync(QString discLabel);

    /**
     * @brief 阻塞式R盘Udf(1.02版本)追加刻录数据操作接口，刻录成功后会立即弹出光盘
     * @param discLabel 卷标名
     * @param burnPath 需刻录文件的路径
     * @return R盘追加刻录是否成功
     */
    bool appendBurnDataUdfSync(QString discLabel, const QString &burnPath);



Q_SIGNALS:
    /**
     * @brief 方法work()获取光盘信息后发出的信号
     * @param pThis
     */
    void workFinished(DiscControl* pThis);

    /**
     * @brief 光盘UDF格式化完成后发出的信号
     * @param bool 格式化成功或失败
     * @param QString 格式化失败的原因；格式化成功返回为空
     */
    void formatUdfFinished(bool, QString);

    /**
     * @brief R盘UDF追加刻录完成后发出的信号
     * @param bool 追加刻录成功或失败
     * @param QString 追加刻录失败的原因；追加刻录成功返回为空
     */
    void appendBurnDataUdfFinished(bool, QString);			  /**< bool为false时，QString表示udf追加刻录失败原因 */

private:
    DiscControlImpl * m_impl;
};

}

#endif // DISCCONTROL_H
